import { PublicKey, SystemProgram, SYSVAR_RENT_PUBKEY, } from "@solana/web3.js";
import { getTokenBridgeDerivedAccounts } from "@certusone/wormhole-sdk/lib/cjs/solana";
import { deriveClaimKey, derivePostedVaaKey, } from "@certusone/wormhole-sdk/lib/cjs/solana/wormhole";
import { getAssociatedTokenAddressSync, TOKEN_PROGRAM_ID, } from "@solana/spl-token";
import { isBytes, parseTokenTransferVaa, } from "@certusone/wormhole-sdk";
import { deriveEndpointKey, deriveMintAuthorityKey, deriveRedeemerAccountKey, deriveTokenBridgeConfigKey, deriveWrappedMetaKey, deriveWrappedMintKey, } from "@certusone/wormhole-sdk/lib/cjs/solana/tokenBridge";
import { bridgeKey, foreignContractKey, redeemerConfigKey, transferKey } from "./pdas";
import { toBN } from "@render-foundation/spl-utils";
export function createReceiveRndrV1Ix(program, payer, tokenBridgeProgramId, wormholeProgramId, wormholeMessage, accts) {
    const parsed = isBytes(wormholeMessage)
        ? parseTokenTransferVaa(wormholeMessage)
        : wormholeMessage;
    const wrappedMint = deriveWrappedMintKey(tokenBridgeProgramId, parsed.tokenChain, parsed.tokenAddress);
    const tokenBridgeAccounts = getCompleteTransferWrappedWithPayloadCpiAccounts(tokenBridgeProgramId, wormholeProgramId, payer, parsed);
    const recipient = new PublicKey(parsed.tokenTransferPayload.subarray(1, 33));
    /*const recipientTokenAccount = getAssociatedTokenAddressSync(
      accts.rndrMint,
      recipient
    )*/
    return program.methods
        .receiveRenderV1({ vaaHash: [...parsed.hash] })
        .accounts({
        config: redeemerConfigKey(program.programId)[0],
        foreignContract: foreignContractKey(parsed.emitterChain)[0],
        recipient,
        transfer: transferKey(parsed.hash)[0],
        rndrMint: accts.rndrMint,
        circuitBreaker: accts.circuitBreakerK,
        bridge: accts.bridgeK,
        payerTokenAccount: getAssociatedTokenAddressSync(wrappedMint, new PublicKey(payer)),
        tokenBridgeProgram: new PublicKey(tokenBridgeProgramId),
        wrndrMint: tokenBridgeAccounts.tokenBridgeWrappedMint,
        ...tokenBridgeAccounts,
    })
        .instruction();
}
// Temporary
export function getCompleteTransferWrappedWithPayloadCpiAccounts(tokenBridgeProgramId, wormholeProgramId, payer, vaa) {
    const parsed = isBytes(vaa) ? parseTokenTransferVaa(vaa) : vaa;
    const mint = deriveWrappedMintKey(tokenBridgeProgramId, parsed.tokenChain, parsed.tokenAddress);
    const cpiProgramId = new PublicKey(parsed.to);
    return {
        payer: new PublicKey(payer),
        tokenBridgeConfig: deriveTokenBridgeConfigKey(tokenBridgeProgramId),
        vaa: derivePostedVaaKey(wormholeProgramId, parsed.hash),
        tokenBridgeClaim: deriveClaimKey(tokenBridgeProgramId, parsed.emitterAddress, parsed.emitterChain, parsed.sequence),
        tokenBridgeForeignEndpoint: deriveEndpointKey(tokenBridgeProgramId, parsed.emitterChain, parsed.emitterAddress),
        tokenBridgeRedeemer: deriveRedeemerAccountKey(cpiProgramId),
        tokenBridgeWrappedMint: mint,
        tokenBridgeWrappedMeta: deriveWrappedMetaKey(tokenBridgeProgramId, mint),
        tokenBridgeMintAuthority: deriveMintAuthorityKey(tokenBridgeProgramId),
        rent: SYSVAR_RENT_PUBKEY,
        systemProgram: SystemProgram.programId,
        tokenProgram: TOKEN_PROGRAM_ID,
        wormholeProgram: new PublicKey(wormholeProgramId),
    };
}
export function createInitializeIx(program, tokenBridgeProgramId, wormholeProgramId, accts, args) {
    const { owner, wRndrMint, rndrMint, bridgeK, circuitBreakerK } = accts;
    const tokenBridgeAccounts = getTokenBridgeDerivedAccounts(program.programId, tokenBridgeProgramId, wormholeProgramId);
    return program.methods
        .initializeBridgeV1({
        ...args,
        incentiveThreshold: toBN(args.incentiveThreshold, rndrMint.decimals)
    })
        .accounts({
        owner: owner,
        bridge: bridgeKey(rndrMint.address, wRndrMint, program.programId)[0],
        rndrMint: rndrMint.address,
        wrndrMint: wRndrMint,
        circuitBreaker: circuitBreakerK,
        redeemerConfig: redeemerConfigKey(program.programId)[0],
        tokenBridgeProgram: new PublicKey(tokenBridgeProgramId),
        wormholeProgram: new PublicKey(wormholeProgramId),
        ...tokenBridgeAccounts,
    }).instruction();
}
export async function createRegisterForeignContractIx(program, payer, tokenBridgeProgramId, chain, contractAddress, tokenBridgeForeignAddress, rndrMint, wrndrMint) {
    return program.methods
        .registerForeignContractV1({ chain, address: [...contractAddress] })
        .accounts({
        owner: new PublicKey(payer),
        rndrMint,
        bridge: bridgeKey(rndrMint, wrndrMint)[0],
        wrndrMint,
        foreignContract: foreignContractKey(chain)[0],
        tokenBridgeForeignEndpoint: deriveEndpointKey(tokenBridgeProgramId, chain, tokenBridgeForeignAddress),
        tokenBridgeProgram: new PublicKey(tokenBridgeProgramId),
    })
        .instruction();
}
//# sourceMappingURL=instructions.js.map